@@ -31,10 +31,10 @@ def register():
     if session_token:
         user, session = user_manager.authenticate_user_session(session_token)
         if user:
-            return redirect(redirect_page, 302)
+            return redirect(redirect_page, 302)  # lgtm [py/url-redirection]
 
     if user_manager.get_user_count() > 0:
-        return redirect('/login?redirect=' + redirect_page, 302)
+        return redirect('/login?redirect=' + redirect_page, 302)  # lgtm [py/url-redirection]
 
     if request.form:
         username = request.form.get('username')
@@ -49,7 +49,7 @@ def register():
                                                        if not remember else None)
 
             if session:
-                redirect_target = redirect(redirect_page, 302)
+                redirect_target = redirect(redirect_page, 302)  # lgtm [py/url-redirection]
                 response = make_response(redirect_target)
                 response.set_cookie('session_token', session.session_token)
                 return response
